@@ -16,8 +16,6 @@
 #include <sys/types.h> 
 #include <netinet/in.h>
 
-/* get this before config.h too. */
-#include <syslog.h>
 #ifdef __APPLE__
 /* need this before arpa/nameser.h */
 #  define BIND_8_COMPAT
@@ -69,6 +67,8 @@
 #include <netinet/ip.h>
 #include <netinet/ip_icmp.h>
 #include <sys/uio.h>
+#include <syslog.h>
+#include <dirent.h>
 #ifndef HAVE_LINUX_NETWORK
 #  include <net/if_dl.h>
 #endif
@@ -110,6 +110,7 @@ extern int capset(cap_user_header_t header, cap_user_data_t data);
 #define OPT_DBUS           524288
 #define OPT_BOOTP_DYNAMIC  1048576
 #define OPT_NO_PING        2097152
+#define OPT_LEASE_RO       4194304
 
 struct all_addr {
   union {
@@ -290,7 +291,8 @@ struct dhcp_lease {
   char *hostname, *fqdn; /* name from client-hostname option or config */
   char auth_name;        /* hostname came from config, not from client */
   char new;              /* newly created */
-  char old;              /* read from leasefile */
+  char changed;          /* modified */
+  char aux_changed;      /* CLID or expiry changed */
   time_t expires;        /* lease expiry */
 #ifdef HAVE_BROKEN_RTC
   unsigned int length;
@@ -319,6 +321,7 @@ struct dhcp_config {
   char *hostname;
   struct dhcp_netid netid;
   struct in_addr addr;
+  time_t decline_time;
   unsigned int lease_time, wildcard_mask;
   struct dhcp_config *next;
 };
@@ -331,6 +334,9 @@ struct dhcp_config {
 #define CONFIG_ADDR             32
 #define CONFIG_NETID            64
 #define CONFIG_NOCLID          128
+#define CONFIG_FROM_ETHERS     256    /* entry created by /etc/ethers */
+#define CONFIG_ADDR_HOSTS      512    /* address added by from /etc/hosts */
+#define CONFIG_DECLINED       1024    /* address declined by client */
 
 struct dhcp_opt {
   int opt, len, flags;
@@ -417,7 +423,8 @@ struct daemon {
   struct iname *if_names, *if_addrs, *if_except, *dhcp_except;
   struct bogus_addr *bogus_addr;
   struct server *servers;
-  int cachesize;
+  int log_fac; /* log facility */
+  int cachesize, ftabsize;
   int port, query_port;
   unsigned long local_ttl;
   struct hostsfile *addn_hosts;
@@ -532,15 +539,15 @@ char *print_mac(struct daemon *daemon, unsigned char *mac, int len);
 struct daemon *read_opts (int argc, char **argv, char *compile_opts);
 
 /* forward.c */
-void forward_init(int first);
 void reply_query(struct serverfd *sfd, struct daemon *daemon, time_t now);
 void receive_query(struct listener *listen, struct daemon *daemon, time_t now);
 unsigned char *tcp_request(struct daemon *daemon, int confd, time_t now,
 			   struct in_addr local_addr, struct in_addr netmask);
+void server_gone(struct daemon *daemon, struct server *server);
 
 /* network.c */
 struct serverfd *allocate_sfd(union mysockaddr *addr, struct serverfd **sfds);
-void reload_servers(char *fname, struct daemon *daemon);
+int reload_servers(char *fname, struct daemon *daemon);
 void check_servers(struct daemon *daemon);
 int enumerate_interfaces(struct daemon *daemon);
 struct listener *create_wildcard_listeners(int port);
