@@ -12,25 +12,15 @@
 
 #include "dnsmasq.h"
 
-static struct frec *frec_list;
+static struct frec *frec_list = NULL;
 
-static struct frec *get_new_frec(time_t now);
+static struct frec *get_new_frec(struct daemon *daemon, time_t now);
 static struct frec *lookup_frec(unsigned short id);
 static struct frec *lookup_frec_by_sender(unsigned short id,
 					  union mysockaddr *addr,
 					  unsigned int crc);
 static unsigned short get_id(void);
 
-/* May be called more than once. */
-void forward_init(int first)
-{
-  struct frec *f;
-  
-  if (first)
-    frec_list = NULL;
-  for (f = frec_list; f; f = f->next)
-    f->new_id = 0;
-}
 
 /* Send a UDP packet with it's source address set as "source" 
    unless nowild is true, when we just send it with the kernel default */
@@ -242,7 +232,7 @@ static void forward_query(struct daemon *daemon, int udpfd, union mysockaddr *ud
       if (gotname)
 	flags = search_servers(daemon, now, &addrp, gotname, daemon->namebuff, &type, &domain);
       
-      if (!flags && !(forward = get_new_frec(now)))
+      if (!flags && !(forward = get_new_frec(daemon, now)))
 	/* table full - server failure. */
 	flags = F_NEG;
       
@@ -847,7 +837,7 @@ unsigned char *tcp_request(struct daemon *daemon, int confd, time_t now,
     }
 }
 
-static struct frec *get_new_frec(time_t now)
+static struct frec *get_new_frec(struct daemon *daemon, time_t now)
 {
   struct frec *f = frec_list, *oldest = NULL;
   time_t oldtime = now;
@@ -880,7 +870,7 @@ static struct frec *get_new_frec(time_t now)
       return oldest;
     }
   
-  if (count > FTABSIZ)
+  if (count > daemon->ftabsize)
     { /* limit logging rate so syslog isn't DOSed either */
       if (!warntime || difftime(now, warntime) > LOGRATE)
 	{
@@ -927,6 +917,21 @@ static struct frec *lookup_frec_by_sender(unsigned short id,
   return NULL;
 }
 
+/* A server record is going away, remove references to it */
+void server_gone(struct daemon *daemon, struct server *server)
+{
+  struct frec *f;
+  
+  for (f = frec_list; f; f = f->next)
+    if (f->new_id != 0 && f->sentto == server)
+      f->new_id = 0;
+  
+  if (daemon->last_server == server)
+    daemon->last_server = NULL;
+
+  if (daemon->srv_save == server)
+    daemon->srv_save = NULL;
+}
 
 /* return unique random ids between 1 and 65535 */
 static unsigned short get_id(void)
